#!/bin/env python
# -*- coding: utf-8 -*-

from optparse import OptionParser, OptionGroup
import os
import subprocess
import json
import re

def nagexit(exitc, statusline, perfdata):
    print "{0}: {1}|{2}".format(
        {0:'OK',1:'WARNING',2:'CRITICAL',3:'UNKNOWN'}[exitc],
        "\n".join(statusline),
        " ".join(perfdata)
    )
    exit(exitc)

def check_iperf3():
    try:
        params = [ "iperf3", "-J", "-c", options.remote, "-p", str(options.port) ]
        if options.downstream:
            params += [ "-R" ]
        raw_data = subprocess.check_output(
            params,
            stderr=subprocess.STDOUT,
            universal_newlines=True
        )
    except OSError as e:
        if e.errno == 2:
            nagexit(3, ["cannot find iperf3"], [])
    except subprocess.CalledProcessError as e:
        nagexit(3, [e.output], [])

    json_data = json.loads(raw_data)
    statusline = []
    perfdata = []
    rc = 0

    if options.rate_crit and options.rate_warn:
        if json_data["end"]["sum_sent"]["bits_per_second"] <= options.rate_crit:
            if rc < 2: rc = 2
            statusline += [ "transfer rate below critical threshold" ]
        elif json_data["end"]["sum_sent"]["bits_per_second"] <= options.rate_warn:
            if rc < 1: rc = 1
            statusline += [ "transfer rate below warning threshold" ]

    if options.retrans_crit and options.retrans_warn:
        if json_data["end"]["sum_sent"]["retransmits"] >= options.retrans_crit:
            if rc < 2: rc = 2
            statusline += [ "retransmissions over critical threshold" ]
        elif json_data["end"]["sum_sent"]["retransmits"] >= options.retrans_warn:
            if rc < 1: rc = 1
            statusline += [ "retransmissions over warning threshold" ]

    i = 0
    for intv in json_data["intervals"]:
        perfdata += [
            "'bps{0}'={1};".format(
                i, intv["sum"]["bits_per_second"]
            )
        ]
        i+=1

    perfdata += [
        "'bps_avg'={0};{1};{2};".format(
            json_data["end"]["sum_sent"]["bits_per_second"],
            options.rate_warn or "",
            options.rate_crit or ""
        ),
        "'retrans_sum'={0};{1};{2};".format(
            json_data["end"]["sum_sent"]["retransmits"],
            options.retrans_warn or "",
            options.retrans_crit or ""
        ),
        "'local_cpu'={}%;".format(
            json_data["end"]["cpu_utilization_percent"]["host_total"]
        ),
        "'remote_cpu'={}%;".format(
            json_data["end"]["cpu_utilization_percent"]["remote_total"]
        )
    ]
    nagexit(rc, statusline, perfdata)

if __name__ == "__main__":
    desc="%prog is used to run an iperf3 check against a given host."
    parser = OptionParser(description=desc,version="%prog version 0.1")
    
    gen_opts = OptionGroup(parser, "Generic options")
    thres_opts = OptionGroup(parser, "Threshold options")
    parser.add_option_group(gen_opts)
    parser.add_option_group(thres_opts)
    
    # transfer rate
    thres_opts.add_option("-w", "--rate-warning", dest="rate_warn",
        type="int", metavar="BITS", action="store",
        help="Defines the transfer rate's warning threshold")
    thres_opts.add_option("-c", "--rate-critical", dest="rate_crit",
        type="int", metavar="BITS", action="store",
        help="Defines the transfer rate's critical threshold")

    # retransmits
    thres_opts.add_option("-W", "--retransmit-warning", dest="retrans_warn",
        type="int", metavar="RETRANS", action="store",
        help="Defines the retransmission warning threshold")
    thres_opts.add_option("-C", "--retransmit-critical", dest="retrans_crit",
        type="int", metavar="RETRANS", action="store",
        help="Defines the retransmission critical threshold")

    # -H / --host
    gen_opts.add_option("-H", "--host", dest="host",
            type="string", action="store",
            help="dummy switch for compatibility reasons")

    # -r / --remote
    gen_opts.add_option("-r", "--remote", dest="remote",
            type="string", action="store",
            help="iperf3 server to connect to")

    # -p / --port
    gen_opts.add_option("-p", "--port", dest="port",
            type="int", action="store", default=5201,
            help="iperf3 server port to connect to")

    # -d / --downstream
    gen_opts.add_option("-d", "--downstream", dest="downstream",
            action="store_true", default=False,
            help="measure downstream instead of upstream")

    (options, args) = parser.parse_args()
    if not options.remote:
        parser.print_help()
        exit(3)

    check_iperf3()
